@@ -1,7 +1,7 @@
 #
 # AWL simulator - Operator translator
 #
-# Copyright 2012-2017 Michael Buesch <m@bues.ch>
+# Copyright 2012-2018 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,9 +32,10 @@ import re
 from awlsim.common.cpuspecs import * #+cimport
 from awlsim.common.cpuconfig import *
 from awlsim.common.datatypehelpers import * #+cimport
+from awlsim.common.exceptions import *
+from awlsim.common.util import *
 
 from awlsim.core.operators import * #+cimport
-from awlsim.core.util import *
 from awlsim.core.parameters import * #+cimport
 from awlsim.core.memory import * #+cimport
 from awlsim.core.offset import * #+cimport
@@ -290,6 +291,8 @@ class AwlOpTranslator(object):
 		self.__mnemonics2constOperTab = self.__getConstOperTable()
 
 	def __translateIndirectAddressing(self, opDesc, rawOps):
+#@cy		cdef uint64_t area_u64
+
 		# rawOps starts _after_ the opening bracket '['
 		try:
 			if rawOps[0].upper() in ("AR1", "AR2"):
@@ -321,9 +324,8 @@ class AwlOpTranslator(object):
 						       offset=None,
 						       insn=opDesc.operator.insn)
 				offsetOp.pointer = offsetPtr
-				try:
-					area = AwlIndirectOpConst.optype2area[opDesc.operator.operType]
-				except KeyError:
+				area = AwlIndirectOpConst.optype2area(opDesc.operator.operType)
+				if area < 0:
 					raise AwlSimError("Invalid memory area type in "
 						"register indirect addressing operator")
 				indirectOp = make_AwlIndirectOp(area=area,
@@ -356,20 +358,20 @@ class AwlOpTranslator(object):
 				if offsetOp.operType == AwlOperatorTypes.INDIRECT:
 					raise AwlSimError("Only direct operators supported "
 						"inside of indirect operator brackets.")
-				try:
-					area = AwlIndirectOpConst.optype2area[opDesc.operator.operType]
-				except KeyError:
+				area = AwlIndirectOpConst.optype2area(opDesc.operator.operType)
+				if area < 0:
 					raise AwlSimError("Invalid memory area type in "
 						"indirect addressing operator")
-				if area == PointerConst.AREA_NONE_S:
+				area_u64 = area
+				if area_u64 == PointerConst.AREA_NONE_S:
 					raise AwlSimError("No memory area code specified in "
 						"indirect addressing operator")
-				if area in (AwlIndirectOpConst.EXT_AREA_T_S,
-					    AwlIndirectOpConst.EXT_AREA_Z_S,
-					    AwlIndirectOpConst.EXT_AREA_BLKREF_DB_S,
-					    AwlIndirectOpConst.EXT_AREA_BLKREF_DI_S,
-					    AwlIndirectOpConst.EXT_AREA_BLKREF_FC_S,
-					    AwlIndirectOpConst.EXT_AREA_BLKREF_FB_S):
+				if area_u64 in (AwlIndirectOpConst.EXT_AREA_T_S,
+						AwlIndirectOpConst.EXT_AREA_Z_S,
+						AwlIndirectOpConst.EXT_AREA_BLKREF_DB_S,
+						AwlIndirectOpConst.EXT_AREA_BLKREF_DI_S,
+						AwlIndirectOpConst.EXT_AREA_BLKREF_FC_S,
+						AwlIndirectOpConst.EXT_AREA_BLKREF_FB_S):
 					expectedOffsetOpWidth = 16
 				else:
 					expectedOffsetOpWidth = 32
@@ -615,7 +617,7 @@ class AwlOpTranslator(object):
 		# Byte array immediate
 		immediate, fields = AwlDataType.tryParseImmediate_ByteArray(rawOps)
 		if immediate is not None:
-			size = 32 if fields == 9 else 16
+			size = 32 if fields == 10 else 16
 			oper = make_AwlOperator(AwlOperatorTypes.IMM, size, None, None)
 			oper.immediate = immediate
 			return OpDescriptor(oper, fields)
