@@ -2,7 +2,7 @@
 #
 # AWL simulator - CPU call stack
 #
-# Copyright 2012-2017 Michael Buesch <m@bues.ch>
+# Copyright 2012-2018 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@ from __future__ import division, absolute_import, print_function, unicode_litera
 from awlsim.common.compat import *
 
 from awlsim.common.exceptions import *
+from awlsim.common.util import *
 
 from awlsim.core.datatypes import *
 from awlsim.core.memory import * #+cimport
@@ -35,7 +36,6 @@ from awlsim.core.blockinterface import *
 from awlsim.core.datablocks import * #+cimport
 from awlsim.core.parameters import * #+cimport
 from awlsim.core.parenstack import * #+cimport
-from awlsim.core.util import *
 
 
 __all__ = [
@@ -68,39 +68,31 @@ class CallStackElem(object): #+cdef
 	# Get an FC interface operand by interface field index.
 	def getInterfIdxOper(self, interfaceFieldIndex): #@nocy
 #@cy	cpdef AwlOperator getInterfIdxOper(self, uint32_t interfaceFieldIndex):
-		try:
-#@cy			if self._interfRefs is None:
-#@cy				raise KeyError
-			return self._interfRefs[interfaceFieldIndex]
-		except (AttributeError, KeyError) as e:
+		if self._interfRefs is None: #+unlikely
 			# Huh, no interface ref? We might have been called via raw call.
 			raise AwlSimError("The block interface field could not "
 				"be found. This probably means that this function block "
 				"has been called with a raw call instruction like UC or CC, "
 				"but the function block has an interface. This is not "
 				"allowed in Awlsim.")
+		return self._interfRefs[interfaceFieldIndex]
 
 	# FB parameter translation:
 	# Translate FB DB-pointer variable.
 	# This is used for FB IN_OUT compound data type parameters.
 	# Returns the actual DB-pointer data. (Not an operator!)
 	def _FB_trans_dbpointer(self, param, rvalueOp): #@nocy
-#@cy	cdef bytearray _FB_trans_dbpointer(self, AwlParamAssign param, AwlOperator rvalueOp):
-#@cy		cdef uint32_t ptr
+#@cy	cdef uint64_t _FB_trans_dbpointer(self, AwlParamAssign param, AwlOperator rvalueOp):
+#@cy		cdef uint64_t dbPtr
 #@cy		cdef int32_t dbNumber
-#@cy		cdef bytearray dbPtrData
 
-		dbPtrData = bytearray(6)
+		dbPtr = 0
 		dbNumber = rvalueOp.offset.dbNumber
 		if dbNumber >= 0:
-			dbPtrData[0] = (dbNumber >> 8) & 0xFF
-			dbPtrData[1] = dbNumber & 0xFF
-		ptr = rvalueOp.makePointerValue()
-		dbPtrData[2] = (ptr >> 24) & 0xFF
-		dbPtrData[3] = (ptr >> 16) & 0xFF
-		dbPtrData[4] = (ptr >> 8) & 0xFF
-		dbPtrData[5] = ptr & 0xFF
-		return dbPtrData
+			dbPtr = dbNumber
+			dbPtr <<= 32
+		dbPtr |= rvalueOp.makePointerValue()
+		return dbPtr
 
 	# FC parameter translation:
 	# Don't perform translation.
@@ -157,7 +149,8 @@ class CallStackElem(object): #+cdef
 #@cy		cdef S7CPU cpu
 #@cy		cdef AwlOffset loffset
 #@cy		cdef int32_t dbNumber
-#@cy		cdef uint64_t area
+#@cy		cdef int64_t area
+#@cy		cdef uint64_t area_u64
 #@cy		cdef AwlOperator storeOper
 #@cy		cdef uint32_t widthMaskAll
 
@@ -176,20 +169,24 @@ class CallStackElem(object): #+cdef
 		else:
 			dbNumber = rvalueOp.offset.dbNumber
 		cpu.store(storeOper,
-			  max(0, dbNumber),
+			  make_AwlMemoryObject_fromScalar16(max(0, dbNumber)),
 			  widthMaskAll)
-		storeOper.offset = loffset + make_AwlOffset(2, 0)
+		storeOper.offset = loffset.addInt(2, 0)
 		storeOper.width = 32
-		area = AwlIndirectOpConst.optype2area[rvalueOp.operType]
-		if area == PointerConst.AREA_L_S:
-			area = PointerConst.AREA_VL_S
-		elif area == PointerConst.AREA_VL_S:
+		area = AwlIndirectOpConst.optype2area(rvalueOp.operType)
+		if area < 0: #+unlikely
+			raise AwlSimBug("FC_trans_dbpointerInVL: Invalid rValueOp area. "
+				"(area=%d, operType=%d)" % (area, rvalueOp.operType))
+		area_u64 = area
+		if area_u64 == PointerConst.AREA_L_S:
+			area_u64 = PointerConst.AREA_VL_S
+		elif area_u64 == PointerConst.AREA_VL_S: #+unlikely
 			raise AwlSimError("Cannot forward VL-parameter "
 					  "to called FC")
-		elif area == PointerConst.AREA_DI_S:
-			area = PointerConst.AREA_DB_S
+		elif area_u64 == PointerConst.AREA_DI_S:
+			area_u64 = PointerConst.AREA_DB_S
 		cpu.store(storeOper,
-			  area | rvalueOp.offset.toPointerValue(),
+			  make_AwlMemoryObject_fromScalar32(area_u64 | rvalueOp.offset.toPointerValue()),
 			  widthMaskAll)
 		# Return the operator for the DB pointer.
 		return make_AwlOperator(AwlOperatorTypes.MEM_VL,
@@ -283,7 +280,7 @@ class CallStackElem(object): #+cdef
 		# Call the operator translation handler (Python)
 		try:							#@nocy
 			trans = self._FC_paramTrans[oper.operType]	#@nocy
-		except KeyError as e:					#@nocy
+		except KeyError as e:					#@nocy #@nocov
 			self._FCTransBug(param, oper)			#@nocy
 		return trans(self, param, oper)				#@nocy
 
@@ -378,7 +375,7 @@ class CallStackElem(object): #+cdef
 #@cy			oper = None
 #@cy		return oper
 
-	def _FCTransError(self, param, oper):
+	def _FCTransError(self, param, oper): #@nocov
 		raise AwlSimError("Do not know how to translate "
 			"FC parameter '%s' for call. The specified "
 			"actual-parameter is not allowed in this call." % (
@@ -450,7 +447,7 @@ class CallStackElem(object): #+cdef
 		# Unlink this call stack element from the previous one.
 		self.prevCse = None
 
-	def __repr__(self):
+	def __repr__(self): #@nocov
 		return "CallStackElem of %s" % str(self.block)
 
 #
@@ -485,6 +482,7 @@ def make_CallStackElem(cpu,						#@nocy
 #@cy	cdef AwlStructField structField
 #@cy	cdef AwlStructInstance structInstance
 #@cy	cdef uint32_t widthMaskAll
+#@cy	cdef AwlMemoryObject memObj
 
 	cse = CallStackElem()
 
@@ -528,8 +526,8 @@ def make_CallStackElem(cpu,						#@nocy
 					   structField.compound:
 						# Compound data type with IN_OUT decl.
 						# Make a DB-ptr to the actual data.
-						data = cse._FB_trans_dbpointer(
-								param, param.rvalueOp)
+						memObj = make_AwlMemoryObject_fromScalar48(
+							cse._FB_trans_dbpointer(param, param.rvalueOp))
 						# Get the DB-ptr struct field.
 						structField = structField.finalOverride
 					else:
@@ -539,13 +537,15 @@ def make_CallStackElem(cpu,						#@nocy
 						if structField.callByRef:
 							# Do not fetch. Type is passed 'by reference'.
 							# This is for TIMER, COUNTER, etc...
-							data = param.rvalueOp.resolve().offset.byteOffset
+							memObj = make_AwlMemoryObject_fromScalar16(
+								param.rvalueOp.resolve(True).offset.byteOffset)
 						else:
-							data = cpu.fetch(param.rvalueOp, widthMaskAll)
+							memObj = cpu.fetch(param.rvalueOp, widthMaskAll)
 					# Transfer data into DBI.
 					structInstance.setFieldData(structField,
-								    data,
+								    memObj,
 								    instanceBaseOffset)
+			cse._interfRefs = None
 		else:
 			# This is a call to an FC.
 			# Prepare the interface (IN/OUT/INOUT) references.
@@ -560,7 +560,7 @@ def make_CallStackElem(cpu,						#@nocy
 				# Call the operator translation handler (Python)
 				try:							#@nocy
 					trans = cse._FC_paramTrans[oper.operType]	#@nocy
-				except KeyError as e:					#@nocy
+				except KeyError as e:					#@nocy #@nocov
 					cse._FCTransError(param, oper)			#@nocy
 				cse._interfRefs[param.interfaceFieldIndex] = trans(	#@nocy
 						cse, param, oper)			#@nocy
@@ -570,6 +570,8 @@ def make_CallStackElem(cpu,						#@nocy
 #@cy				if oper is None:
 #@cy					cse._FCTransError(param, oper)
 #@cy				cse._interfRefs[param.interfaceFieldIndex] = oper
+	else:
+		cse._interfRefs = None
 
 	# Prepare the localdata stack.
 	cpu.activeLStack.enterStackFrame()
