@@ -27,8 +27,8 @@ from awlsim.common.datatypehelpers import * #+cimport
 from awlsim.common.immutable import *
 from awlsim.common.enumeration import *
 from awlsim.common.exceptions import *
+from awlsim.common.util import *
 
-from awlsim.core.util import *
 from awlsim.core.timers import * #+cimport
 from awlsim.core.offset import * #+cimport
 from awlsim.core.identifier import *
@@ -328,7 +328,8 @@ class AwlDataType(OptionalImmutable):
 			if len(nameTokens) < 2:
 				raise AwlSimError("Invalid '%s' block data type" %\
 					nameTokens[0])
-			blockNumber = cls.tryParseImmediate_INT(nameTokens[1])
+			blockNumber = cls.tryParseImmediate_INT(nameTokens[1],
+								maxLim=0xFFFF)
 			if blockNumber is None:
 				raise AwlSimError("Invalid '%s' block data type "\
 					"index" % nameTokens[0])
@@ -547,34 +548,45 @@ class AwlDataType(OptionalImmutable):
 		value = None
 		if tokens is None:
 			value = 0
-		elif len(tokens) == 9:
+		elif len(tokens) == 10:
 			if typeId == self.TYPE_DWORD:
 				value, fields = self.tryParseImmediate_ByteArray(
 							tokens)
-		elif len(tokens) == 5:
+		elif len(tokens) == 6:
 			if typeId == self.TYPE_WORD:
 				value, fields = self.tryParseImmediate_ByteArray(
 							tokens)
-			elif typeId == self.TYPE_DT:
+		elif len(tokens) == 5:
+			if typeId == self.TYPE_DT:
 				value = self.tryParseImmediate_DT(tokens)
 			elif typeId == self.TYPE_TOD:
 				value = self.tryParseImmediate_TOD(tokens)
 		elif len(tokens) == 2:
 			if typeId == self.TYPE_TIMER:
 				if tokens[0].upper() == "T":
-					value = self.tryParseImmediate_INT(tokens[1])
+					value = self.tryParseImmediate_INT(tokens[1],
+									   minLim=0,
+									   maxLim=0xFFFF)
 			elif typeId == self.TYPE_COUNTER:
 				if tokens[0].upper() in ("C", "Z"):
-					value = self.tryParseImmediate_INT(tokens[1])
+					value = self.tryParseImmediate_INT(tokens[1],
+									   minLim=0,
+									   maxLim=0xFFFF)
 			elif typeId == self.TYPE_BLOCK_DB:
 				if tokens[0].upper() == "DB":
-					value = self.tryParseImmediate_INT(tokens[1])
+					value = self.tryParseImmediate_INT(tokens[1],
+									   minLim=0,
+									   maxLim=0xFFFF)
 			elif typeId == self.TYPE_BLOCK_FB:
 				if tokens[0].upper() == "FB":
-					value = self.tryParseImmediate_INT(tokens[1])
+					value = self.tryParseImmediate_INT(tokens[1],
+									   minLim=0,
+									   maxLim=0xFFFF)
 			elif typeId == self.TYPE_BLOCK_FC:
 				if tokens[0].upper() == "FC":
-					value = self.tryParseImmediate_INT(tokens[1])
+					value = self.tryParseImmediate_INT(tokens[1],
+									   minLim=0,
+									   maxLim=0xFFFF)
 		elif len(tokens) == 1:
 			if typeId == self.TYPE_BOOL:
 				value = self.tryParseImmediate_BOOL(
@@ -671,10 +683,10 @@ class AwlDataType(OptionalImmutable):
 		return None
 
 	@classmethod
-	def tryParseImmediate_INT(cls, token):
+	def tryParseImmediate_INT(cls, token, minLim=-32768, maxLim=32767):
 		try:
 			immediate = int(token, 10)
-			if immediate > 32767 or immediate < -32768:
+			if immediate > maxLim or immediate < minLim:
 				raise AwlSimError("16-bit immediate overflow")
 		except ValueError:
 			return None
@@ -687,8 +699,7 @@ class AwlDataType(OptionalImmutable):
 			return None
 		try:
 			immediate = int(token[2:], 10)
-			if immediate > 2147483647 or immediate < -2147483648: #@nocy
-#@cy			if immediate > 2147483647LL or immediate < -2147483648LL:
+			if immediate > 2147483647 or immediate < -2147483648: #+suffix-LL
 				raise AwlSimError("32-bit immediate overflow")
 			immediate &= 0xFFFFFFFF
 		except ValueError as e:
@@ -825,7 +836,16 @@ class AwlDataType(OptionalImmutable):
 			# Strings are supposed to be traditional
 			# "latin1" encoding compatible and the resulting
 			# bytes are "latin1" encoded bytes.
-			data = token.encode(AwlSource.COMPAT_ENCODING)
+			if isMicroPython:
+				trans = pivotDict(AwlSource.latin1Trans)
+				data = bytearray()
+				for c in token:
+					if c in trans:
+						data += trans[c]
+					else:
+						data += c.encode(AwlSource.COMPAT_ENCODING)
+			else:
+				data = token.encode(AwlSource.COMPAT_ENCODING)
 			data = bytearray(data)
 			if len(data) != len(token):
 				raise ValueError
@@ -1144,30 +1164,29 @@ class AwlDataType(OptionalImmutable):
 
 	@classmethod
 	def tryParseImmediate_ByteArray(cls, tokens):
-		tokens = [ t.upper() for t in tokens ]
-		if not tokens[0].startswith("B#("):
+		if tokens[0].upper() != "B#" or tokens[1] != "(":
 			return None, None
 		try:
-			if len(tokens) >= 5 and\
-			   tokens[2] == ',' and\
-			   tokens[4] == ')':
-				fields = 5
-				a, b = int(tokens[1], 10),\
-				       int(tokens[3], 10)
+			if len(tokens) >= 6 and\
+			   tokens[3] == ',' and\
+			   tokens[5] == ')':
+				fields = 6
+				a, b = int(tokens[2], 10),\
+				       int(tokens[4], 10)
 				if a < 0 or a > 0xFF or\
 				   b < 0 or b > 0xFF:
 					raise ValueError
 				immediate = (a << 8) | b
-			elif len(tokens) >= 9 and\
-			     tokens[2] == ',' and\
-			     tokens[4] == ',' and\
-			     tokens[6] == ',' and\
-			     tokens[8] == ')':
-				fields = 9
-				a, b, c, d = int(tokens[1], 10),\
-					     int(tokens[3], 10),\
-					     int(tokens[5], 10),\
-					     int(tokens[7], 10)
+			elif len(tokens) >= 10 and\
+			     tokens[3] == ',' and\
+			     tokens[5] == ',' and\
+			     tokens[7] == ',' and\
+			     tokens[9] == ')':
+				fields = 10
+				a, b, c, d = int(tokens[2], 10),\
+					     int(tokens[4], 10),\
+					     int(tokens[6], 10),\
+					     int(tokens[8], 10)
 				if a < 0 or a > 0xFF or\
 				   b < 0 or b > 0xFF or\
 				   c < 0 or c > 0xFF or\
