@@ -2,7 +2,7 @@
 #
 # AWL simulator - System-blocks
 #
-# Copyright 2012-2017 Michael Buesch <m@bues.ch>
+# Copyright 2012-2018 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@ from __future__ import division, absolute_import, print_function, unicode_litera
 from awlsim.common.compat import *
 
 from awlsim.common.exceptions import *
+from awlsim.common.util import *
 
 from awlsim.core.instructions.insn_generic_call import * #+cimport
 from awlsim.core.systemblocks.error_codes import *
@@ -60,13 +61,13 @@ class SystemBlock(StaticCodeBlock): #+cdef
 
 	# Fetch the value of a block-interface field.
 	def fetchInterfaceFieldByName(self, name): #@nocy
-#@cy	cdef object fetchInterfaceFieldByName(self, object name):
+#@cy	cdef AwlMemoryObject fetchInterfaceFieldByName(self, object name) except NULL:
 		return self.cpu.fetch(self.__interfaceOpers[name],
 				      self.__widthMaskAll)
 
 	# Store a value to a block-interface field.
 	def storeInterfaceFieldByName(self, name, value): #@nocy
-#@cy	cdef storeInterfaceFieldByName(self, object name, object value):
+#@cy	cdef storeInterfaceFieldByName(self, object name, AwlMemoryObject value):
 		return self.cpu.store(self.__interfaceOpers[name], value,
 				      self.__widthMaskAll)
 
