@@ -66,8 +66,9 @@ class FupCompiler_ElemFactory(XmlFactory):
 				subType = tag.getAttr("subtype", None)
 				content = tag.getAttr("content", None)
 				uuid = tag.getAttr("uuid", None)
+				enabled = tag.getAttrBool("enabled", True)
 				self.elem = FupCompiler_Elem.parse(self.grid,
-					x, y, elemType, subType, content, uuid)
+					x, y, elemType, subType, content, uuid, enabled)
 				if not self.elem:
 					raise self.Error("Failed to parse element "
 						"type '%s'%s at x=%d y=%d" % (
@@ -86,6 +87,9 @@ class FupCompiler_ElemFactory(XmlFactory):
 						self.elem.subElems = self.subelemsFakeGrid.elems.copy()
 						for e in self.elem.subElems:
 							e.grid = self.grid
+							e.x += self.elem.x
+							e.y += self.elem.y
+							e.parentElem = self.elem
 						self.subelemsFakeGrid = None
 					self.grid.addElem(self.elem)
 				self.inElem = False
@@ -108,6 +112,7 @@ class FupCompiler_Elem(FupCompiler_BaseObj):
 	TYPE_MOVE		= EnumGen.item
 	TYPE_CONV		= EnumGen.item
 	TYPE_COUNT		= EnumGen.item
+	TYPE_TIME		= EnumGen.item
 	TYPE_ARITH		= EnumGen.item
 	TYPE_SHIFT		= EnumGen.item
 	TYPE_CMP		= EnumGen.item
@@ -121,6 +126,7 @@ class FupCompiler_Elem(FupCompiler_BaseObj):
 		"move"		: TYPE_MOVE,
 		"convert"	: TYPE_CONV,
 		"counter"	: TYPE_COUNT,
+		"timer"		: TYPE_TIME,
 		"arithmetic"	: TYPE_ARITH,
 		"shift"		: TYPE_SHIFT,
 		"compare"	: TYPE_CMP,
@@ -139,19 +145,20 @@ class FupCompiler_Elem(FupCompiler_BaseObj):
 		"""
 		elems = tuple(elems)
 		try:
-			yShift = max(e.x for e in elems).bit_length()
+			yShift = bit_length(max(e.x for e in elems))
 		except ValueError:
 			return []
 		return sorted(elems,
 			      key=lambda e: (e.y << yShift) + e.x)
 
 	@classmethod
-	def parse(cls, grid, x, y, elemType, subType, content, uuid):
+	def parse(cls, grid, x, y, elemType, subType, content, uuid, enabled):
 		from awlsim.fupcompiler.elembool import FupCompiler_ElemBool
 		from awlsim.fupcompiler.elemoper import FupCompiler_ElemOper
 		from awlsim.fupcompiler.elemmove import FupCompiler_ElemMove
 		from awlsim.fupcompiler.elemconv import FupCompiler_ElemConv
 		from awlsim.fupcompiler.elemcount import FupCompiler_ElemCount
+		from awlsim.fupcompiler.elemtime import FupCompiler_ElemTime
 		from awlsim.fupcompiler.elemarith import FupCompiler_ElemArith
 		from awlsim.fupcompiler.elemshift import FupCompiler_ElemShift
 		from awlsim.fupcompiler.elemcmp import FupCompiler_ElemCmp
@@ -165,6 +172,7 @@ class FupCompiler_Elem(FupCompiler_BaseObj):
 				cls.TYPE_MOVE		: FupCompiler_ElemMove,
 				cls.TYPE_CONV		: FupCompiler_ElemConv,
 				cls.TYPE_COUNT		: FupCompiler_ElemCount,
+				cls.TYPE_TIME		: FupCompiler_ElemTime,
 				cls.TYPE_ARITH		: FupCompiler_ElemArith,
 				cls.TYPE_SHIFT		: FupCompiler_ElemShift,
 				cls.TYPE_CMP		: FupCompiler_ElemCmp,
@@ -180,6 +188,7 @@ class FupCompiler_Elem(FupCompiler_BaseObj):
 						       subType=subType,
 						       content=content)
 				elem.uuid = uuid
+				elem.enabled = enabled
 				return elem
 		except KeyError:
 			pass
@@ -196,6 +205,7 @@ class FupCompiler_Elem(FupCompiler_BaseObj):
 		self.connections = set()		# FupCompiler_Conn
 		self.virtual = bool(virtual)		# Flag: Virtual element
 		self.subElems = set()			# Set of sub-elements (if any)
+		self.parentElem = None			# The parent element (if any)
 
 		# This dict contains the values of the connections,
 		# if this element has already been compiled.
@@ -270,9 +280,12 @@ class FupCompiler_Elem(FupCompiler_BaseObj):
 
 	def connIsOptional(self, conn):
 		"""Check if a connection is optional.
-		The default implementation always returns False.
+		The default implementation returns False, if the element is enabled.
+		The default implementation returns True, if the element is disabled.
 		"""
-		return False
+		if self.enabled:
+			return False
+		return True
 
 	def getConnType(self, conn, preferVKE=False):
 		"""Get the type of a connection.
@@ -388,6 +401,8 @@ class FupCompiler_Elem(FupCompiler_BaseObj):
 		The default implementation raises an exception.
 		Override this method, if required.
 		"""
+		if not self.enabled:
+			return []
 		raise FupElemError("It is not known how to "
 			"compile the connection %s of element %s. "
 			"That probably means it's not allowed to "
@@ -438,6 +453,8 @@ class FupCompiler_Elem(FupCompiler_BaseObj):
 		"""Main element preprocessor entry point.
 		Do not override this. Override _doPreprocess instead.
 		"""
+		if not self.enabled:
+			return
 		self.compileState = self.COMPILE_PREPROCESSING
 
 		# Preprocess the elements connected to the in-connections first.
@@ -455,6 +472,8 @@ class FupCompiler_Elem(FupCompiler_BaseObj):
 		"""Main element compiler entry point.
 		Do not override this. Override _doCompile instead.
 		"""
+		if not self.enabled:
+			return []
 		self.compileState = self.COMPILE_RUNNING
 		result = self._doCompile()
 		self.compileState = self.COMPILE_DONE
@@ -505,14 +524,14 @@ class FupCompiler_Elem(FupCompiler_BaseObj):
 		"""
 		return self.grid.compiler.newInsn_SRD(self, *args, **kwargs)
 
-	def __repr__(self):
+	def __repr__(self): #@nocov
 		return "FupCompiler_Elem(grid, x=%d, y=%d, elemType=%d, "\
 			"subType=%s, content=\"%s\", virtual=%s)" % (
 			self.x, self.y, self.elemType,
 			str(self.subType), self.content,
 			str(self.virtual))
 
-	def toStr(self, extra=()):
+	def toStr(self, pre="", extra=(), post=""): #@nocov
 		values = []
 		if self.x >= 0 and self.y >= 0:
 			values.append("x%d/y%d" % (
@@ -524,8 +543,10 @@ class FupCompiler_Elem(FupCompiler_BaseObj):
 			content = self.content.strip().replace('\n', ' ').replace('\r', '')
 			values.append("'%s'" % content)
 		values.append(shortUUID(self.uuid))
+		if not self.enabled:
+			values.append("disabled")
 		values.extend(extra)
-		return "%s(%s)" % (self.ELEM_NAME, ", ".join(values))
+		return "%s%s(%s)%s" % (pre, self.ELEM_NAME, ", ".join(values), post)
 
 	def __str__(self):
 		return self.toStr()
