@@ -2,7 +2,7 @@
 #
 # AWL simulator - FUP draw widget
 #
-# Copyright 2016-2018 Michael Buesch <m@bues.ch>
+# Copyright 2016-2020 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@ from awlsim.common.compat import *
 
 from awlsim.common.xmlfactory import *
 
+from awlsim.gui.validatorsched import *
 from awlsim.gui.icons import *
 from awlsim.gui.util import *
 from awlsim.gui.fup.fup_grid import *
@@ -41,6 +42,8 @@ class FupContextMenu(QMenu):
 	paste = Signal()
 	edit = Signal()
 	remove = Signal()
+	enable = Signal()
+	disable = Signal()
 	invertConn = Signal()
 	addInput = Signal()
 	addOutput = Signal()
@@ -70,6 +73,12 @@ class FupContextMenu(QMenu):
 						self.__paste)
 		self.__actDel = self.addAction(getIcon("doc_close"),
 					       "&Remove element", self.__del)
+		self.__actEnable = self.addAction(getIcon("enable"),
+						  "E&nable element",
+						  self.__enable)
+		self.__actDisable = self.addAction(getIcon("disable"),
+						   "&Disable element",
+						   self.__disable)
 		self.addSeparator()
 		self.__actInvertConn = self.addAction(getIcon("doc_edit"),
 						      "In&vert connection",
@@ -110,6 +119,12 @@ class FupContextMenu(QMenu):
 	def __del(self):
 		self.remove.emit()
 
+	def __enable(self):
+		self.enable.emit()
+
+	def __disable(self):
+		self.disable.emit()
+
 	def __invertConn(self):
 		self.invertConn.emit()
 
@@ -143,6 +158,12 @@ class FupContextMenu(QMenu):
 	def enableRemove(self, en=True):
 		self.__actDel.setVisible(en)
 
+	def enableEnable(self, en=True):
+		self.__actEnable.setVisible(en)
+
+	def enableDisable(self, en=True):
+		self.__actDisable.setVisible(en)
+
 	def enableInvertConn(self, en=True):
 		self.__actInvertConn.setVisible(en)
 
@@ -193,17 +214,27 @@ class FupDrawWidget(QWidget):
 	#	The parameters are the X/Y pixel coordinates.
 	moveTipChanged = Signal(int, int)
 
-	def __init__(self, parent, interfWidget):
+	# Signal: Emitted, whenever the element or cell selection changes.
+	#	Parameter 0: Boolean, some cells are selected.
+	#	Parameter 1: Boolean, some elements are selected.
+	#	Parameter 2: The grid with the selections.
+	selectionChanged = Signal(bool, bool, FupGrid)
+
+	def __init__(self, parent, interfWidget, getSymTabSourcesFunc):
 		"""parent => Parent QWidget().
-		interfWidget => AwlInterfWidget() instance
+		interfWidget => AwlInterfWidget() instance.
+		getSymTabSourcesFunc => Function to get SymTabSource list.
 		"""
 		QWidget.__init__(self, parent)
 
 		self.__interfWidget = interfWidget
+		self.__getSymTabSourcesFunc = getSymTabSourcesFunc
 		self.__repaintBlocked = Blocker()
 
 		self.__contextMenu = FupContextMenu(self)
 		self.__contextMenu.remove.connect(self.removeElems)
+		self.__contextMenu.enable.connect(lambda: self.enableElems(enable=True))
+		self.__contextMenu.disable.connect(lambda: self.enableElems(enable=False))
 		self.__contextMenu.edit.connect(self.editElems)
 		self.__contextMenu.copy.connect(self.clipboardCopy)
 		self.__contextMenu.cut.connect(self.clipboardCut)
@@ -217,6 +248,7 @@ class FupDrawWidget(QWidget):
 
 		self.__bgBrush = QBrush(QColor("#F5F5F5"))
 		self.__bgSelBrush = QBrush(QColor("#F2F25A"))
+		self.__bgErrBrush = QBrush(QColor("#FF0000"))
 		self.__gridPen = QPen(QColor("#E0E0E0"))
 		self.__gridPen.setWidth(1)
 		self.__lightTextPen = QColor("#808080")
@@ -280,6 +312,14 @@ class FupDrawWidget(QWidget):
 		"""
 		return self.__interfWidget.interfDef
 
+	@property
+	def symTabSources(self):
+		"""Get a list of SymTabSource()s.
+		"""
+		if self.__getSymTabSourcesFunc:
+			return self.__getSymTabSourcesFunc()
+		return []
+
 	def getFont(self, size=8, bold=False, scale=True):
 		"""Get a font.
 		"""
@@ -347,11 +387,21 @@ class FupDrawWidget(QWidget):
 		if not self.__repaintBlocked:
 			QWidget.repaint(self)
 
-	def __contentChanged(self):
+	def __contentChanged(self, clearErrors=True):
+		if clearErrors:
+			self.__grid.clearAllCellErrors()
 		self.__dynGridExpansion()
 		self.repaint()
+		self.__selectionChanged()
 		self.diagramChanged.emit()
 
+	def __selectionChanged(self):
+		grid = self.__grid
+		if grid:
+			self.selectionChanged.emit(bool(grid.selectedCells),
+						   bool(grid.selectedElems),
+						   grid)
+
 	@property
 	def grid(self):
 		return self.__grid
@@ -388,6 +438,14 @@ class FupDrawWidget(QWidget):
 					self.removeElem(elem)
 			self.__contentChanged()
 
+	def enableElems(self, elems=None, enable=True):
+		if self.__grid:
+			if not elems:
+				elems = self.__grid.selectedElems
+			for elem in elems:
+				elem.enabled = enable
+			self.__contentChanged()
+
 	def moveElem(self, elem, toGridX, toGridY,
 		     relativeCoords=False,
 		     checkOnly=False,
@@ -522,6 +580,16 @@ class FupDrawWidget(QWidget):
 					  cellWidth - 1, cellHeight - 1,
 					  r, r)
 
+		# Draw the erroneous cells.
+		p.setBrush(self.__bgErrBrush)
+		p.setPen(Qt.NoPen)
+		r = 5
+		for gridX, gridY in grid.erroneousCells:
+			p.drawRoundedRect(gridX * cellWidth + 1,
+					  gridY * cellHeight + 1,
+					  cellWidth - 1, cellHeight - 1,
+					  r, r)
+
 		# Draw the elements
 		def drawElems(wantForeground, wantCollisions):
 			prevX, prevY = 0, 0
@@ -683,6 +751,7 @@ class FupDrawWidget(QWidget):
 				self.__selectStartCell = (gridX, gridY)
 				self.__selectEndPix = None
 				eventHandled()
+			self.__selectionChanged()
 
 		# Handle middle button press
 		if event.button() == Qt.MidButton:
@@ -713,11 +782,16 @@ class FupDrawWidget(QWidget):
 			self.__contextMenu.enableAddOutput(False)
 			self.__contextMenu.enableRemoveConn(False)
 			self.__contextMenu.enableDisconnWire(False)
+			self.__contextMenu.enableEnable(
+				any(not e.enabled for e in grid.selectedElems))
+			self.__contextMenu.enableDisable(
+				any(e.enabled for e in grid.selectedElems))
 			for i in range(self.__contextMenu.NR_CUSTOM):
 				self.__contextMenu.enableCustomAction(i, False)
 			if elem:
 				elem.prepareContextMenu(self.__contextMenu, area, conn)
-			self.__contextMenu.exec_(self.mapToGlobal(event.pos()))
+			self.__contextMenu.exec_(self.mapToGlobal(event.pos()) + QPoint(3, 3))
+			self.__selectionChanged()
 
 		if not event.isAccepted():
 			QWidget.mousePressEvent(self, event)
@@ -741,6 +815,7 @@ class FupDrawWidget(QWidget):
 			self.__selectStartPix = None
 			self.__selectStartCell = None
 			self.__selectEndPix = None
+			self.__selectionChanged()
 			eventHandled()
 
 		# Handle end of element dragging
@@ -813,6 +888,7 @@ class FupDrawWidget(QWidget):
 			grid.selectElemsInRect(startGridX, startGridY,
 					       gridX, gridY, clear=clear)
 			self.selectTipChanged.emit(x, y)
+			self.__selectionChanged()
 			eventHandled()
 
 		# Handle element dragging
@@ -879,6 +955,7 @@ class FupDrawWidget(QWidget):
 		if not elem:
 			grid.selectCell(gridX, gridY)
 			grid.deselectAllElems()
+		self.__selectionChanged()
 
 		# Handle left button double click
 		if event.button() == Qt.LeftButton:
@@ -921,10 +998,7 @@ class FupDrawWidget(QWidget):
 	def wheelEvent(self, ev):
 		if ev.modifiers() & Qt.ControlModifier:
 			# Ctrl + Scroll-wheel: Zoom
-			if isQt4:
-				numDegrees = ev.delta() / 8
-			else:
-				numDegrees = ev.angleDelta().y() / 8
+			numDegrees = ev.angleDelta().y() / 8
 			numSteps = numDegrees / 15
 
 			self.__wheelSteps += numSteps
@@ -941,34 +1015,44 @@ class FupDrawWidget(QWidget):
 		QWidget.wheelEvent(self, ev)
 
 	def keyPressEvent(self, event):
-		if event.matches(QKeySequence.Delete):
-			self.removeElems()
-			event.accept()
-			return
-		elif isQt5 and (event.matches(QKeySequence.Cancel) or\
-				event.matches(QKeySequence.Deselect)):
-			self.__grid.deselectAllElems()
-			self.repaint()
-			event.accept()
-			return
-		elif event.matches(QKeySequence.SelectAll):
-			for elem in self.__grid.elems:
-				self.__grid.selectElem(elem)
-			self.repaint()
-			event.accept()
-			return
-		elif event.matches(QKeySequence.Copy):
-			self.clipboardCopy()
-			event.accept()
-			return
-		elif event.matches(QKeySequence.Cut):
-			self.clipboardCut()
-			event.accept()
-			return
-		elif event.matches(QKeySequence.Paste):
-			self.clipboardPaste()
-			event.accept()
-			return
+		grid = self.__grid
+		if grid:
+			if event.matches(QKeySequence.Delete):
+				self.removeElems()
+				event.accept()
+				return
+			elif (event.matches(QKeySequence.Cancel) or
+			      event.matches(QKeySequence.Deselect)):
+				self.__grid.deselectAllElems()
+				self.__selectionChanged()
+				self.repaint()
+				event.accept()
+				return
+			elif event.matches(QKeySequence.SelectAll):
+				for elem in self.__grid.elems:
+					self.__grid.selectElem(elem)
+				self.__selectionChanged()
+				self.repaint()
+				event.accept()
+				return
+			elif event.matches(QKeySequence.Copy):
+				self.clipboardCopy()
+				event.accept()
+				return
+			elif event.matches(QKeySequence.Cut):
+				self.clipboardCut()
+				event.accept()
+				return
+			elif event.matches(QKeySequence.Paste):
+				self.clipboardPaste()
+				event.accept()
+				return
+
+			if event.key() in (Qt.Key_Enter, Qt.Key_Return):
+				if len(grid.selectedElems) == 1:
+					self.editElems()
+					event.accept()
+					return
 
 		QWidget.keyPressEvent(self, event)
 
@@ -1066,8 +1150,7 @@ class FupDrawWidget(QWidget):
 		if not mime.hasFormat("application/x-awlsim-xml-fup-elem"):
 			return ignore()
 		mimeData = mime.data("application/x-awlsim-xml-fup-elem")
-		if isQt4:
-			mimeData = mimeData.data() # QByteArray to bytes/str
+		mimeData = mimeData.data() # QByteArray to bytes/str
 		mimeData = bytearray(mimeData)
 		if not mimeData:
 			return ignore()
@@ -1217,3 +1300,27 @@ class FupDrawWidget(QWidget):
 				"Please select another position in the grid.")
 		self.__contentChanged()
 		return True
+
+	def handleAwlSimError(self, e):
+		"""Handle an AwlSimError.
+		This tries to make exception root cause visible in the diagram.
+		If e is None, then reset all error markers.
+		"""
+		grid = self.__grid
+		if not grid:
+			return
+		if e:
+			coord = e.getCoordinates()
+			if coord:
+				x, y = coord[0], coord[1]
+				hadError = grid.cellHasError(x, y)
+				grid.setCellError(x, y, False)
+				otherError = grid.haveErroneousCells()
+				grid.clearAllCellErrors()
+				grid.setCellError(x, y)
+				if not hadError or otherError:
+					self.__contentChanged(clearErrors=False)
+		else:
+			# No error. Clear all marked cells.
+			if grid.haveErroneousCells():
+				self.__contentChanged(clearErrors=True)
