@@ -6,12 +6,14 @@ exec apt install \
 	devscripts \
 	git \
 	pypy \
+	pypy-coverage \
 	pypy-dev \
 	python3 \
 	python3-cffi \
+	python3-coverage \
 	python3-dev \
-	python3-nose \
 	python3-pyqt5 \
-	python3-pyside \
-	python3-serial \
-	time
+	python3-pyside2.qtcore \
+	python3-pyside2.qtgui \
+	python3-pyside2.qtwidgets \
+	python3-serial
