@@ -2,7 +2,7 @@
 #
 # PROFIBUS DP - Master
 #
-# Copyright (c) 2013-2019 Michael Buesch <m@bues.ch>
+# Copyright (c) 2013-2020 Michael Buesch <m@bues.ch>
 #
 # Licensed under the terms of the GNU General Public License version 2,
 # or (at your option) any later version.
@@ -15,8 +15,14 @@ from pyprofibus.fdl import *
 from pyprofibus.dp import *
 from pyprofibus.util import *
 
+import gc
 import math
 
+__all__ = [
+	"DpSlaveDesc",
+	"DPM1",
+	"DPM2",
+]
 
 class DpSlaveState(object):
 	"""Run time state of a DP slave that is managed by a DPM instance.
@@ -50,6 +56,24 @@ class DpSlaveState(object):
 		STATE_DX	: 0.5,
 	}
 
+	__slots__ = (
+		"__nextState",
+		"__prevState",
+		"__state",
+		"__stateTimeout",
+		"dxStartTime",
+		"faultDeb",
+		"fcb",
+		"inData",
+		"master",
+		"outData",
+		"pendingReq",
+		"pendingReqTimeout",
+		"rxQueue",
+		"shortAckReceived",
+		"slaveDesc",
+	)
+
 	def __init__(self, master, slaveDesc):
 		self.master = master
 		self.slaveDesc = slaveDesc
@@ -131,13 +155,32 @@ class DpSlaveDesc(object):
 	is managed by a DPM instance.
 	"""
 
+	__slots__ = (
+		"index",
+		"name",
+		"chkCfgTelegram",
+		"dpm",
+		"gsd",
+		"identNumber",
+		"setPrmTelegram",
+		"setPrmTelegram",
+		"slaveAddr",
+		"slaveConf",
+		"userData",
+	)
+
 	def __init__(self,
 		     gsd,
-		     slaveAddr):
+		     slaveAddr,
+		     slaveConf=None):
 		self.gsd = gsd
 		self.slaveAddr = slaveAddr
+		self.slaveConf = slaveConf
 		self.identNumber = gsd.getIdentNumber() if gsd else 0
 		self.dpm = None
+		self.name = None
+		self.index = None
+		self.userData = {} # For use by application code.
 
 		# Prepare a Set_Prm telegram.
 		self.setPrmTelegram = DpTelegram_SetPrm_Req(
@@ -221,16 +264,38 @@ class DpSlaveDesc(object):
 		return self.dpm.getSlaveInData(self)
 
 	def __repr__(self):
-		return "DPSlaveDesc(identNumber=%s, slaveAddr=%d)" %\
+		return "DpSlaveDesc(identNumber=%s, slaveAddr=%d)" %\
 			(intToHex(self.identNumber), self.slaveAddr)
 
 class DpMaster(object):
+	__slots__ = (
+		"__runTimer",
+		"__runCount",
+		"__haveToken",
+		"__runNextSlaveIndex",
+		"__slaveDescs",
+		"__slaveDescsList",
+		"__slaveStates",
+		"__slowDown",
+		"__slowDownFact",
+		"__slowDownUntil",
+		"debug",
+		"dpTrans",
+		"dpmClass",
+		"fdlTrans",
+		"masterAddr",
+		"phy",
+	)
+
 	def __init__(self, dpmClass, phy, masterAddr, debug=False):
 		self.dpmClass = dpmClass
 		self.phy = phy
 		self.masterAddr = masterAddr
 		self.debug = debug
 
+		self.__runTimer = monotonic_time()
+		self.__runCount = 0
+
 		# Create the transceivers
 		self.fdlTrans = FdlTransceiver(self.phy)
 		self.dpTrans = DpTransceiver(self.fdlTrans, thisIsMaster=True)
@@ -464,6 +529,10 @@ class DpMaster(object):
 						self.__errorMsg("Slave %d reports a faulty "
 							"parameterization (Set_Prm)." %\
 							slave.slaveDesc.slaveAddr)
+					if telegram.prmReq():
+						self.__debugMsg("Slave %d requests a new "
+							"parameterization (Set_Prm)." %\
+							slave.slaveDesc.slaveAddr)
 					if telegram.isNotSupp():
 						self.__errorMsg("Slave %d replied with "
 							"\"function not supported\". "
@@ -478,9 +547,9 @@ class DpMaster(object):
 						self.__debugMsg("Slave %d diagnostic "
 							"always-one-bit is zero." %\
 							slave.slaveDesc.slaveAddr)
-
 					if telegram.hasExtDiag():
 						pass#TODO turn on red DIAG-LED
+
 					if telegram.isReadyDataEx():
 						slave.setState(slave.STATE_DX)
 						return None
@@ -524,8 +593,8 @@ class DpMaster(object):
 					slave.faultDeb.fault()
 					continue
 				resFunc = telegram.fc & FdlTelegram.FC_RESFUNC_MASK
-				if resFunc in {FdlTelegram.FC_DH,
-					       FdlTelegram.FC_RDH}:
+				if resFunc in (FdlTelegram.FC_DH,
+					       FdlTelegram.FC_RDH,):
 					self.__debugMsg("Slave %d requested diagnostics." %\
 						slave.slaveDesc.slaveAddr)
 					slave.setState(slave.STATE_WDXRDY, 0.2)
@@ -646,6 +715,17 @@ class DpMaster(object):
 	def run(self):
 		"""Run the DP-Master state machine.
 		"""
+		if self.debug:
+			self.__runCount += 1
+			now = monotonic_time()
+			if now >= self.__runTimer + 10.0:
+				cps = self.__runCount / (now - self.__runTimer)
+				self.__debugMsg("State machine calls: "
+						"%.1f /s = %.3f s/call" % (
+						cps, 1.0 / cps))
+				self.__runTimer = now
+				self.__runCount = 0
+
 		if self.__slowDown:
 			# Master slowdown is active.
 			# Do not run state machine until the end of the slowdown.
@@ -689,6 +769,8 @@ class DpMaster(object):
 		# Initialize the RX filter
 		self.fdlTrans.setRXFilter([self.masterAddr,
 					   FdlTelegram.ADDRESS_MCAST])
+		# Free memory
+		gc.collect()
 
 	def __syncFreezeHelper(self, groupMask, controlCommand):
 		slave = self.__slaveStates[FdlTelegram.ADDRESS_MCAST]
