@@ -9,12 +9,16 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+from pyprofibus.compat import *
 
 from pyprofibus.gsd.parser import GsdParser, GsdError
 from pyprofibus.dp import DpCfgDataElement
 
 import difflib
 
+__all__ = [
+	"GsdInterp",
+]
 
 class GsdInterp(GsdParser):
 	"""GSD file/data interpreter.
@@ -164,13 +168,13 @@ class GsdInterp(GsdParser):
 		def trunc(data, length, fieldname, extend = True):
 			if length is not None:
 				if extend:
-					data.extend((0,) * (length - len(data)))
+					data.extend(bytearray((0,) * (length - len(data))))
 				if len(data) > length:
 					self.__interpWarn("User_Prm_Data "
 						"truncated by %s" % fieldname)
 					data[:] = data[0:length]
 		# Get the global data.
-		data = self.getField("User_Prm_Data", bytearray())
+		data = bytearray(self.getField("User_Prm_Data", b""))
 		trunc(data, self.getField("User_Prm_Data_Len"),
 		      "User_Prm_Data_Len")
 		for dataConst in self.getField("Ext_User_Prm_Data_Const", []):
@@ -200,7 +204,7 @@ class GsdInterp(GsdParser):
 			self.__interpWarn("DPv1 User_Prm_Data override ignored")
 		trunc(data, self.getField("Max_User_Prm_Data_Len"),
 		      "Max_User_Prm_Data_Len", False)
-		return data
+		return bytes(data)
 
 	def getIdentNumber(self):
 		"""Get the ident number.
@@ -249,7 +253,7 @@ class GsdInterp(GsdParser):
 		if order:
 			text.append("Order number:      %s\n" % order)
 
-		for module in self.getField("Module"):
+		for module in self.getField("Module", []):
 			if module.getField("Preset"):
 				continue
 			text.append("Available module:  \"%s\"\n" % module.name)
